--[[----------------------------------------------------------------------------
  Copyright (c) 2008, Tom Wieland
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of idQuestAutomation nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
------------------------------------------------------------------------------]]

local _G = _G
local addon = {}

addon.quests = {}
addon.eventframe = CreateFrame('Frame')

function addon:stripText (t) -- thx Shadowed
	if not t then return end
	t = t:gsub('%[.*%]%s*','')
	t = t:gsub('|c%x%x%x%x%x%x%x%x(.+)|r','%1')
	t = t:gsub('(.+) %(.+%)', '%1')
	t = t:trim()
	return t
end

function addon:GOSSIP_SHOW (event)
	if IsShiftKeyDown() then return end

	local button
	local text

	if GossipFrame.buttonIndex then
		for i = 1, GossipFrame.buttonIndex do
			button = _G['GossipTitleButton'..i]
			text = self:stripText(button:GetText())

			if button:IsVisible() then
				if button.type == 'Available' then
					button:Click()
				elseif button.type == 'Active' then
					if self.quests[text] then
						self.quests[text] = nil
						button:Click()
					end
				end
			end
		end
	end
end

function addon:QUEST_GREETING (event)
	if IsShiftKeyDown() then return end

	local button
	local text

	for i = 1, 38 do
		button = _G['QuestTitleButton'..i]

		if button then
			text = self:stripText(button:GetText())

			if button:IsVisible() then
				if button.isActive == 1 then
					if self.quests[text] then
						self.quests[text] = nil
						button:Click()
					end
				else
					button:Click()
				end
			end
		end
	end
end

function addon:QUEST_DETAIL (event)
	if IsShiftKeyDown() then return end
	AcceptQuest()
end

function addon:QUEST_PROGRESS (event)
	if IsShiftKeyDown() then return end
	if IsQuestCompletable() then
		CompleteQuest()
	end
end

function addon:QUEST_COMPLETE (event)
	if IsShiftKeyDown() then return end
	if GetNumQuestChoices() <= 1 then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function addon:QUEST_LOG_UPDATE (event)
	local startingQuestLogSelection = GetQuestLogSelection()
	local numEntries, numQuests = GetNumQuestLogEntries()
	local q

	if numEntries > 0 then
		for i = 1, numEntries do
			SelectQuestLogEntry(i)
			q = {GetQuestLogTitle(i)}
			if q and q[7] then
				self.quests[q[1]] = true
			end
		end
	end

	SelectQuestLogEntry(startingQuestLogSelection)
end

function addon.OnEvent (frame, event, ...)
	if event == 'GOSSIP_SHOW' then
		addon:GOSSIP_SHOW(...)
	elseif event == 'QUEST_GREETING' then
		addon:QUEST_GREETING(...)
	elseif event == 'QUEST_DETAIL' then
		addon:QUEST_DETAIL(...)
	elseif event == 'QUEST_PROGRESS' then
		addon:QUEST_PROGRESS(...)
	elseif event == 'QUEST_COMPLETE' then
		addon:QUEST_COMPLETE(...)
	elseif event == 'QUEST_LOG_UPDATE' then
		addon:QUEST_LOG_UPDATE(...)
	end
end

addon.eventframe:SetScript('OnEvent', addon.OnEvent)
addon.eventframe:RegisterEvent('GOSSIP_SHOW')
addon.eventframe:RegisterEvent('QUEST_GREETING')
addon.eventframe:RegisterEvent('QUEST_DETAIL')
addon.eventframe:RegisterEvent('QUEST_PROGRESS')
addon.eventframe:RegisterEvent('QUEST_COMPLETE')
addon.eventframe:RegisterEvent('QUEST_LOG_UPDATE')

_G.idQuestAutomation = addon
